"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  ServiceContext: () => ServiceContext,
  chainSideEffects: () => chainSideEffects,
  isErrorResponse: () => isErrorResponse,
  patchFn: () => patchFn,
  queryKey: () => queryKey,
  queryOptions: () => queryOptions,
  sideEffect: () => sideEffect,
  updateFn: () => updateFn,
  useServiceContext: () => useServiceContext,
  useServiceMutation: () => useServiceMutation,
  useServiceQuery: () => useServiceQuery
});
module.exports = __toCommonJS(src_exports);

// src/queryKey.ts
function queryKey(method, req) {
  if (req === void 0) return [method.name];
  return [method.name, req];
}

// src/serviceContext.ts
var import_react = require("react");
var ServiceContext = (0, import_react.createContext)({});
var useServiceContext = () => {
  return (0, import_react.useContext)(ServiceContext);
};

// src/sideEffect.ts
var import_react_query = require("@tanstack/react-query");
function sideEffect(m, options) {
  const queryClient = (0, import_react_query.useQueryClient)();
  const { mapKey = (x) => x, patchFn: patch, updateFn: update, invalidate } = options;
  return {
    onMutate: async (req) => {
      const key = queryKey(m, mapKey ? mapKey(req) : void 0);
      await queryClient.cancelQueries({ queryKey: key });
      let originalData;
      queryClient.setQueryData(key, (oldData) => {
        originalData = oldData;
        return patch ? patch(oldData, req) : oldData;
      });
      return { [key.join("-")]: originalData };
    },
    onSuccess: async (result, req, _context) => {
      const key = queryKey(m, mapKey ? mapKey(req) : void 0);
      if (update) {
        queryClient.setQueryData(key, (oldData) => update(oldData, result));
      }
      if (invalidate === "remove") {
        queryClient.removeQueries({ queryKey: key });
      } else if (invalidate) {
        await queryClient.invalidateQueries({
          queryKey: key,
          refetchType: typeof invalidate == "string" ? invalidate : "active"
        });
      }
    },
    onError: (_error, req, context) => {
      const key = queryKey(m, mapKey ? mapKey(req) : void 0);
      const originalData = context[key.join("-")];
      queryClient.setQueryData(key, originalData);
    }
  };
}
function chainSideEffects(...mutations) {
  return {
    onMutate: async (req) => {
      const context = {};
      for (const mutation of mutations) {
        Object.assign(context, await mutation?.onMutate(req));
      }
      return context;
    },
    onSuccess: async (result, req, context) => {
      for (const mutation of mutations) {
        await mutation?.onSuccess(result, req, context);
      }
    },
    onError: async (error, req, context) => {
      for (const mutation of mutations) {
        await mutation?.onError(error, req, context);
      }
    }
  };
}
function patchFn(oldData, update) {
  return { ...oldData, ...update };
}
function updateFn(_oldData, result) {
  return result;
}

// src/types.ts
function isErrorResponse(error) {
  return error.code !== void 0;
}

// src/useServiceMutation.ts
var import_react_query2 = require("@tanstack/react-query");
var import_react2 = require("react");
function useServiceMutation(method, options) {
  const reqCtx = (0, import_react2.useContext)(ServiceContext);
  return (0, import_react_query2.useMutation)({
    ...options,
    mutationFn: (req) => {
      const resp = method(req, {
        ...reqCtx,
        headers: {
          "Content-Type": "application/json",
          ...reqCtx.headers
        }
      });
      return resp;
    }
  });
}

// src/useServiceQuery.ts
var import_react_query3 = require("@tanstack/react-query");
var import_react3 = require("react");
function useServiceQuery(method, req, options) {
  const reqCtx = (0, import_react3.useContext)(ServiceContext);
  return (0, import_react_query3.useQuery)(queryOptions(method, req, reqCtx, options));
}
function queryOptions(method, req, reqInit, options) {
  return {
    ...options,
    queryKey: options?.queryKey ?? queryKey(method, req),
    queryFn: () => {
      const resp = method(req, {
        ...reqInit,
        headers: {
          "Content-Type": "application/json",
          ...reqInit?.headers
        }
      });
      if (options?.onError) {
        return resp.catch(options.onError);
      }
      return resp;
    }
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ServiceContext,
  chainSideEffects,
  isErrorResponse,
  patchFn,
  queryKey,
  queryOptions,
  sideEffect,
  updateFn,
  useServiceContext,
  useServiceMutation,
  useServiceQuery
});
//# sourceMappingURL=index.cjs.map